"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildApp = void 0;
const lowcode_builder_1 = require("@cloudbase/lowcode-builder");
const fs_extra_1 = __importDefault(require("fs-extra"));
const chalk_1 = __importDefault(require("chalk"));
const common_1 = require("../../common");
const path_1 = __importDefault(require("path"));
const DIST_PATH = './dist';
const DEFAULT_INPUTS = {
    _inputFile: 'input.json',
    debug: false,
    runtime: process.env.CLOUDBASE_CIID ? lowcode_builder_1.RUNTIME.CI : lowcode_builder_1.RUNTIME.NONE,
    appId: 'test',
    buildTypeList: [lowcode_builder_1.BuildType.WEB],
    generateMpType: 'app',
    subAppSerializeDataList: [],
    dependencies: [],
    extraData: { isComposite: false, compProps: {} },
    mpAppId: '',
    deployOptions: {
        mode: (process.env.deployMode || lowcode_builder_1.DEPLOY_MODE.PREVIEW),
        publicPath: '',
    },
    ignoreInstall: false,
};
class Builder {
    constructor(api, inputs) {
        this.api = api;
        this.logger = api.logger || common_1.Logger;
        let inputJSONPath = path_1.default.resolve(api.projectPath, inputs._inputFile || DEFAULT_INPUTS._inputFile);
        let params = fs_extra_1.default.existsSync(inputJSONPath) ? fs_extra_1.default.readJsonSync(inputJSONPath) : {};
        this._resolvedInputs = this.normalizInputs(api, resolveInputs(inputs, resolveInputs(params, DEFAULT_INPUTS)));
        if (fs_extra_1.default.existsSync(inputJSONPath)) {
            fs_extra_1.default.writeFileSync(inputJSONPath, JSON.stringify(this._resolvedInputs, undefined, 2));
        }
    }
    normalizInputs(api, inputs) {
        var _a, _b, _c, _d;
        if (!inputs.mainAppSerializeData) {
            throw new Error('缺少必须参数: mainAppSerializeData');
        }
        const { buildTypeList = [] } = inputs;
        /**
         * 过滤处理 dependencies
         */
        let map = {};
        inputs.dependencies = (inputs.dependencies || []).reduce((list, item) => {
            if (!map[`${item.name}@${item.version}`]) {
                map[`${item.name}@${item.version}`] = true;
                list.push(item);
            }
            return list;
        }, []);
        inputs.mainAppSerializeData.extra = {
            ...inputs.mainAppSerializeData.extra,
            envId: ((_a = inputs.mainAppSerializeData.extra) === null || _a === void 0 ? void 0 : _a.envId) || api.envId,
        };
        if (!inputs.mainAppSerializeData.mpPkgUrl) {
            if (buildTypeList.includes(lowcode_builder_1.BuildType.APP) ||
                buildTypeList.includes(lowcode_builder_1.BuildType.XPAGE_PC) ||
                buildTypeList.includes(lowcode_builder_1.BuildType.ADMIN_PORTAL)) {
                inputs.mainAppSerializeData.extra.historyType = lowcode_builder_1.HISTORY_TYPE.HASH;
            }
        }
        if ((0, lowcode_builder_1.buildAsWebByBuildType)(buildTypeList)) {
            const { appId, mainAppSerializeData, deployOptions } = inputs;
            let { appConfig = {} } = mainAppSerializeData.extra;
            let { window = {} } = appConfig;
            let path = this._getWebRootPath({ appId, deployOptions });
            inputs.mainAppSerializeData.extra.appConfig = {
                ...appConfig,
                window: {
                    ...window,
                    // Todo: 处理自定义域名逻辑
                    publicPath: (0, lowcode_builder_1.buildAsXPageByBuildType)(buildTypeList)
                        ? ((_b = inputs.deployOptions) === null || _b === void 0 ? void 0 : _b.publicPath) || `https://${(_c = inputs.mainAppSerializeData.extra) === null || _c === void 0 ? void 0 : _c.domain}${path}`
                        : path,
                    basename: (0, lowcode_builder_1.buildAsAdminPortalByBuildType)(buildTypeList)
                        ? `app/${appId}${((_d = inputs.deployOptions) === null || _d === void 0 ? void 0 : _d.mode) !== lowcode_builder_1.DEPLOY_MODE.UPLOAD ? '-preview' : ''}`
                        : (0, lowcode_builder_1.buildAsXPageByBuildType)(buildTypeList)
                            ? '/'
                            : path,
                },
            };
            inputs.subAppSerializeDataList = [];
            // web 构建不处理小程序插件
            if (inputs.mainAppSerializeData.extra.miniprogramPlugins) {
                inputs.mainAppSerializeData.extra.miniprogramPlugins = [];
            }
        }
        else {
            // 小程序构建
            const { mpAppId, mpDeployPrivateKey, deployOptions = {} } = inputs;
            inputs.deployOptions = {
                ...deployOptions,
                mpAppId: deployOptions.mpAppId || mpAppId,
                mpDeployPrivateKey: deployOptions.mpDeployPrivateKey || mpDeployPrivateKey,
            };
            if (inputs.deployOptions.targetMpAppId === undefined) {
                inputs.deployOptions.targetMpAppId = inputs.deployOptions.mpAppId;
            }
        }
        if (inputs.extraData.isComposite) {
            Object.keys(inputs.extraData.compProps.events).forEach((eName) => {
                inputs.extraData.compProps.events[eName] = `$$EVENT_${eName}$$`;
            });
        }
        return inputs;
    }
    _getWebRootPath({ appId, deployOptions }) {
        return (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.mode) === lowcode_builder_1.DEPLOY_MODE.PREVIEW ? `/${appId}/preview/` : `/${appId}/production/`;
    }
    async build(data = undefined) {
        var _a;
        this.logger.info('👉 开始生成文件...');
        const outputPath = path_1.default.resolve(this.api.projectPath, DIST_PATH);
        fs_extra_1.default.emptyDirSync(outputPath);
        if (!data) {
            let { appId, mpAppId, mainAppSerializeData, subAppSerializeDataList, dependencies, runtime, buildTypeList, generateMpType, deployOptions, extraData = { isComposite: false, compProps: {} }, ignoreInstall, } = this._resolvedInputs;
            data = {
                cals: mainAppSerializeData,
                subAppCalsList: subAppSerializeDataList,
                dependencies,
                appKey: appId,
                runtime,
                ignoreInstall,
                buildTypeList,
                mode: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.mode) === lowcode_builder_1.DEPLOY_MODE.UPLOAD ? lowcode_builder_1.WebpackModeType.PRODUCTION : lowcode_builder_1.WebpackModeType.NONE,
                deployOptions,
                generateMpType,
                // isCleanDistDir: false,
                plugins: [],
                extraData,
                resourceAppId: mpAppId,
                domain: (_a = mainAppSerializeData.extra) === null || _a === void 0 ? void 0 : _a.domain,
                output: { path: outputPath },
            };
        }
        const appPath = await (0, lowcode_builder_1.buildWedaApp)(data);
        this.logger.info(`👉 生成到目录成功：${chalk_1.default.bold.yellow(appPath)}`);
        return appPath;
    }
}
function resolveInputs(inputs, defaultInputs) {
    return Object.assign({}, defaultInputs, inputs);
}
async function buildApp(ctx, params) {
    var _a, _b, _c;
    const builder = new Builder(ctx, params);
    const appDir = await builder.build();
    if (fs_extra_1.default.existsSync(path_1.default.join(ctx.projectPath, '.env'))) {
        await fs_extra_1.default.copyFile(path_1.default.join(ctx.projectPath, '.env'), path_1.default.join(appDir, '.env'));
    }
    let cloudbaseRc = {
        version: '2.0',
        envId: '{{env.ENV_ID}}',
        lowcode: {
            app: {
                deploy: {},
            },
        },
    };
    const currentCloudRcPath = path_1.default.join(ctx.projectPath, 'cloudbaserc.json');
    if (fs_extra_1.default.existsSync(currentCloudRcPath)) {
        const current = await fs_extra_1.default.readJSON(currentCloudRcPath);
        cloudbaseRc = {
            ...cloudbaseRc,
            ...current,
            lowcode: {
                ...current.lowcode,
                app: {
                    deploy: {
                        projectPath: path_1.default.join(appDir, 'preview'),
                    },
                },
            },
        };
        await fs_extra_1.default.writeFile(currentCloudRcPath, JSON.stringify({
            ...current,
            lowcode: {
                ...current.lowcode,
                app: {
                    ...(_a = current.lowcode) === null || _a === void 0 ? void 0 : _a.app,
                    deploy: {
                        ...(_c = (_b = current.lowcode) === null || _b === void 0 ? void 0 : _b.app) === null || _c === void 0 ? void 0 : _c.deploy,
                        projectPath: path_1.default.join(appDir, 'preview'),
                    },
                },
            },
        }, undefined, 2));
    }
    await fs_extra_1.default.writeFile(path_1.default.join(appDir, 'cloudbaserc.json'), JSON.stringify(cloudbaseRc, undefined, 2));
    return appDir;
}
exports.buildApp = buildApp;
