"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.gracePublishComps = exports.GracePublishService = void 0;
const toolbox_1 = require("@cloudbase/toolbox");
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const lodash_1 = __importDefault(require("lodash"));
const constant_1 = require("./constant");
const graceBuild_1 = require("../graceBuild");
const common_1 = require("../../common");
const PlatformMap = {
    web: {
        alias: 'H5',
        sourceAlias: 'source',
        sourcePath: './.temp/sourceWeb',
        sourceZippedFileName: './.temp/sourceWeb.zip',
    },
    mp: {
        alias: '小程序',
        sourceAlias: 'mpSource',
        sourcePath: './.temp/sourceMp',
        sourceZippedFileName: './.temp/sourceMp.zip',
    },
};
class GracePublishService extends graceBuild_1.GraceBuildService {
    constructor(info) {
        super(info);
        this.publishInfo = info;
        this.cloudService = toolbox_1.CloudApiService.getInstance('lowcode');
        this.lcdsConfig = {
            components: {},
            actions: {},
            schemaVersion: common_1.COMPONENTS_CONFIG_SCHEMA_VERSION,
        };
        this.pkgVersion = fs_extra_1.default.readJsonSync(path_1.default.join(__dirname, '../../../package.json')).version;
        (0, common_1.initLowcodeCloudApiEnv)();
    }
    async getApiName(publicApiName) {
        var _a;
        if (!this.publishInfo.isAdmin)
            return publicApiName;
        try {
            if (!this.privateApiMap) {
                this.privateApiMap = await fs_extra_1.default.readJSON(path_1.default.join(toolbox_1.cloudbaseConfigDir, 'lowcode-private-api.json'), {
                    encoding: 'utf8',
                });
            }
            return (_a = this.privateApiMap[publicApiName]) !== null && _a !== void 0 ? _a : publicApiName;
        }
        catch (_) {
            return publicApiName;
        }
    }
    async publish() {
        // 校验
        this.compsId = await this.validateComps();
        // 打包
        await this.build();
        // 读取最新的配置
        await this.loadLcdsConfig();
        // 上传
        await this.setCloudCompsConfig();
        await this.uploadDistFiles();
        await this.uploadSrcFiles();
        await this.uploadIcon();
    }
    async validateComps() {
        this.info.logger.info('组件库 - 校验组件库信息');
        const { name: compsName } = this.info;
        // 读取云端组件库列表
        const methodName = await this.getApiName('ListUserCompositeGroups');
        const res = await this.cloudService.request(methodName, {
            pageSize: 100,
        });
        const comps = res === null || res === void 0 ? void 0 : res.data;
        if (!(comps === null || comps === void 0 ? void 0 : comps.count)) {
            throw new toolbox_1.CloudBaseError(`云端不存在组件库 ${compsName}，请到低码控制台新建该组件库！`);
        }
        // 校验组件库信息
        const comp = comps.rows.find((row) => row.groupName === compsName);
        if (!comp) {
            throw new toolbox_1.CloudBaseError(`云端不存在组件库 ${compsName}，请到低码控制台新建该组件库！`);
        }
        // todo 校验组件的NAME
        return comp.id;
    }
    async loadLcdsConfig() {
        const { context, name } = this.info;
        const resolveConfigPath = path_1.default.resolve(context, this.outputPath, `${name}.lcds.js`);
        const lcdsConfig = require((0, common_1.pathEscape)(resolveConfigPath))[`@weapps-materials-lcds-${name}`].lcds;
        this.lcdsConfig = lcdsConfig;
    }
    async setCloudCompsConfig() {
        this.info.logger.info('组件库 - 配置云端组件库的信息');
        const methodName = await this.getApiName('UploadComposites');
        await this.cloudService.request(methodName, {
            groupName: this.info.name,
            metas: Object.entries(this.lcdsConfig.components).map(([compName, component]) => {
                var _a, _b, _c, _d, _e;
                const result = {
                    name: compName,
                    title: (_a = component.meta) === null || _a === void 0 ? void 0 : _a.title,
                    description: (_b = component.meta) === null || _b === void 0 ? void 0 : _b.description,
                    category: (_c = component.meta) === null || _c === void 0 ? void 0 : _c.category,
                    platform: Object.keys(this.info.platformRoot).map((key) => PlatformMap[key].alias),
                    // @ts-expect-error TODO
                    configMeta: component === null || component === void 0 ? void 0 : component.configMeta,
                };
                if ((_d = component.meta) === null || _d === void 0 ? void 0 : _d.icon) {
                    result.icon = component.meta.icon;
                }
                if ((_e = component.meta) === null || _e === void 0 ? void 0 : _e.componentOrder) {
                    result.componentOrder = component.meta.componentOrder;
                }
                return result;
            }),
            groupContent: {
                actions: Object.entries(this.lcdsConfig.actions).map(([actionName, action]) => action),
                sourceMap: constant_1.CloudSourceMap,
                manifestMap: constant_1.CloudManifestMap,
            },
        });
    }
    async uploadDistFiles() {
        if (!this.compsId) {
            throw new toolbox_1.CloudBaseError('组件库 - 读取不到组件库云端标识！');
        }
        this.info.logger.info('组件库 - 上传构建产物');
        const { name, context } = this.info;
        // 注意入口文件要和ManifestMap对应;
        let localDistFileNames = await fs_extra_1.default.readdir(path_1.default.resolve(context, this.outputPath));
        localDistFileNames = localDistFileNames.filter((file) => file.startsWith(name) && (file.endsWith('.js') || file.endsWith('.css')));
        // 拼装本地文件META
        const fileMetas = localDistFileNames
            .filter((fileName) => {
            return fs_extra_1.default.existsSync(path_1.default.join(this.outputPath, fileName)) && !fileName.endsWith('.lcds.js');
        })
            .map((fileName, index) => {
            const filePath = path_1.default.join(this.outputPath, fileName);
            const fileContent = fs_extra_1.default.readFileSync(filePath);
            let fileNameKey;
            switch (fileName) {
                case `${name}.control.js`:
                    fileNameKey = constant_1.CloudManifestMap.controlJs;
                    break;
                case `${name}.main.css`:
                    fileNameKey = constant_1.CloudManifestMap.mainCss;
                    break;
                case `${name}.main.js`:
                    fileNameKey = constant_1.CloudManifestMap.mainJs;
                    break;
                default:
                    fileNameKey = path_1.default.join('build/dist/', fileName);
            }
            return {
                fileName: fileNameKey,
                fileContent,
            };
        });
        // 获取上传TOKEN
        const methodName = await this.getApiName('ListCompositeGroupPublicFileAuthorization');
        const res = await this.cloudService.request(methodName, {
            comGroupId: this.compsId,
            files: fileMetas.map(({ fileName, fileContent }) => ({
                fileName,
                contentLength: fileContent.length,
                headers: {
                    'x-cos-meta-cli-version': this.pkgVersion,
                },
            })),
        });
        const tokens = res.data;
        // 上传本地文件
        await Promise.all(tokens.map((token) => {
            const { fileContent } = fileMetas.find(({ fileName }) => fileName === token.id) || {};
            return this.uploadFileToCos(token, fileContent);
        }));
    }
    async uploadSrcFiles() {
        if (!this.compsId) {
            throw new toolbox_1.CloudBaseError('组件库 - 读取不到组件库云端标识！');
        }
        this.info.logger.info('组件库 - 上传源码');
        const { context, platformRoot, configRoot } = this.info;
        let ignoreGlobs = [];
        try {
            // 读取package.json 的 files
            const packageJson = await fs_extra_1.default.readJSON(path_1.default.resolve(context, 'package.json'));
            const files = packageJson.files || [];
            // 过滤感叹号开头的项
            ignoreGlobs = files.filter((file) => file.startsWith('!')).map((file) => file.slice(1));
        }
        catch (e) {
            ignoreGlobs = [];
        }
        // 压缩源码
        await Promise.all(Object.entries(PlatformMap).map(async ([platform, platformInfo]) => {
            const { sourcePath, sourceZippedFileName } = platformInfo;
            const codeRoot = this.getPlatformEntry(platformRoot[platform]);
            // 拷贝源码
            if (!!codeRoot) {
                if (fs_extra_1.default.existsSync(path_1.default.resolve(context, codeRoot))) {
                    await fs_extra_1.default.copy(path_1.default.resolve(context, codeRoot), path_1.default.resolve(context, sourcePath, platform === 'mp' ? 'src' : codeRoot));
                }
                if (platform === 'web') {
                    if (fs_extra_1.default.existsSync(path_1.default.resolve(context, configRoot))) {
                        await fs_extra_1.default.copy(path_1.default.resolve(context, configRoot), path_1.default.resolve(context, sourcePath, configRoot));
                    }
                }
                await this.generatePkg(platform, platformInfo, codeRoot);
            }
            // 写下配置文件
            await fs_extra_1.default.writeJSON(path_1.default.resolve(context, sourcePath, 'meta.json'), this.lcdsConfig, { spaces: 2 });
            await (0, toolbox_1.zipDir)(path_1.default.resolve(context, sourcePath), path_1.default.resolve(context, sourceZippedFileName), [
                'node_modules/**',
                '**/node_modules/**',
                '.DS_Store',
                '**/.DS_Store',
                ...ignoreGlobs,
            ]);
        }));
        // 获取上传TOKEN
        const fileMetas = Object.entries(PlatformMap).map(([platform, platformInfo]) => {
            const { sourceAlias, sourceZippedFileName } = platformInfo;
            const filePath = path_1.default.resolve(context, sourceZippedFileName);
            const fileContent = fs_extra_1.default.readFileSync(filePath);
            return {
                fileName: constant_1.CloudSourceMap[sourceAlias],
                fileContent,
            };
        });
        const methodName = await this.getApiName('ListCompositeGroupPrivateFileAuthorization');
        const res = await this.cloudService.request(methodName, {
            comGroupId: this.compsId,
            files: fileMetas.map(({ fileName, fileContent }) => ({
                fileName,
                contentLength: fileContent.length,
                headers: {
                    'x-cos-meta-cli-version': this.pkgVersion,
                },
            })),
        });
        const tokens = res.data;
        // 上传本地文件
        await Promise.all(tokens.map((token) => {
            const { fileContent } = fileMetas.find(({ fileName }) => fileName === token.id) || {};
            return this.uploadFileToCos(token, fileContent);
        }));
    }
    async uploadIcon() {
        this.info.logger.info('组件库 - 上传图标');
        // 根据meta.icon配置，收集Icon
        const { context, configRoot } = this.info;
        const icons = Object.keys(this.lcdsConfig.components)
            .map((name) => {
            const icon = lodash_1.default.get(this.lcdsConfig.components, `${name}.meta.icon`, '');
            const iconName = icon.split(/\//).pop();
            const resolveIconPath = path_1.default.join(context, configRoot, 'icons', iconName);
            // 文件不存在则返回
            if (!fs_extra_1.default.existsSync(resolveIconPath)) {
                return null;
            }
            // 不符合后缀名字则返回
            const iconExtName = path_1.default.extname(resolveIconPath);
            if (constant_1.ICON_EXT_NAME.indexOf(iconExtName) === -1) {
                return null;
            }
            return {
                name: `build/meta/${name}/icon${iconExtName}`,
                content: fs_extra_1.default.readFileSync(`${resolveIconPath}`),
            };
        })
            .filter((icon) => !!icon);
        // 上传Icon
        const methodName = await this.getApiName('ListCompositeGroupPublicFileAuthorization');
        const res = await this.cloudService.request(methodName, {
            comGroupId: this.compsId,
            files: icons.map((icon) => {
                var _a;
                return ({
                    fileName: icon === null || icon === void 0 ? void 0 : icon.name,
                    contentLength: (_a = icon === null || icon === void 0 ? void 0 : icon.content) === null || _a === void 0 ? void 0 : _a.length,
                    headers: {
                        'x-cos-meta-cli-version': this.pkgVersion,
                    },
                });
            }),
        });
        const tokens = res.data;
        await Promise.all(tokens.map((token) => {
            const { content } = icons.find((icon) => (icon === null || icon === void 0 ? void 0 : icon.name) === token.id) || {};
            return this.uploadFileToCos(token, content);
        }));
        // 更新Icon地址
        const metas = Object.entries(this.lcdsConfig.components).map(([name, component]) => {
            var _a, _b, _c, _d;
            return {
                name,
                title: (_a = component.meta) === null || _a === void 0 ? void 0 : _a.title,
                description: (_b = component.meta) === null || _b === void 0 ? void 0 : _b.description,
                category: (_c = component.meta) === null || _c === void 0 ? void 0 : _c.category,
                platform: Object.keys(this.info.platformRoot).map((key) => PlatformMap[key].alias),
                icon: (_d = tokens.find((token) => token.id.includes(name))) === null || _d === void 0 ? void 0 : _d.url,
            };
        });
        const methodName2 = await this.getApiName('UploadComposites');
        await this.cloudService.request(methodName2, {
            groupName: this.info.name,
            metas,
        });
    }
    uploadFileToCos(token, fileContent) {
        return (0, toolbox_1.putFetch)(token.url, {
            headers: {
                'content-length': String(fileContent.length),
                'x-cos-security-token': token.sessionToken,
                Authorization: token.authorization,
                'x-cos-meta-cli-version': this.pkgVersion,
            },
            body: fileContent,
        });
    }
}
exports.GracePublishService = GracePublishService;
async function gracePublishComps(info) {
    const service = new GracePublishService({ ...info, command: 'publish' });
    await service.publish();
}
exports.gracePublishComps = gracePublishComps;
